﻿using System.Collections.Generic;
using System.Linq;

namespace uBIMEarthTools.Model
{
    /// <summary>
    /// 钻孔数据
    /// </summary>
    public class Borehole
    {
        /// <summary>
        /// 构造函数
        /// </summary>
        /// <param name="name">钻孔编号</param>
        /// <param name="valueList">分层数据</param>
        public Borehole(string name, List<GeologyLayer> valueList)
        {
            Name = name;
            ValueList = valueList;
            StartIndex = 0;
            CurrentType = string.Empty;
        }

        /// <summary>
        /// 构造函数
        /// </summary>
        public Borehole()
        {
            Name = string.Empty;
            ValueList = new List<GeologyLayer>();
            StartIndex = 0;
            CurrentType = string.Empty;
        }

        /// <summary>
        /// 构造函数
        /// </summary>
        public Borehole(Borehole borehole)
        {
            Name = borehole.Name;
            ValueList = borehole.ValueList;
            StartIndex = 0;
            EndIndex = borehole.ValueList.Count - 1;
            CurrentType = string.Empty;
        }

        /// <summary>
        /// 钻孔编号
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// 钻孔分层数据
        /// </summary>
        internal List<GeologyLayer> ValueList { get; set; }

        /// <summary>
        /// 当前地层索引
        /// </summary>
        public int StartIndex { get; set; }
        public int EndIndex { get; set; }

        /// <summary>
        /// 当前地层类型
        /// </summary>
        public string CurrentType { get; set; }

        public void Interpolation(double plus)
        {
            var order = this.ValueList.OrderByDescending(x => x.Point.Z).ToList();
            List<GeologyLayer> newList = new List<GeologyLayer>();
            for (int i = 0; i < order.Count - 1; i++)
            {
                newList.Add(order[i]);
                var z = order[i].Point.Z;
                while (z - plus > order[i + 1].Point.Z)
                {
                    z = z - plus;
                    GeologyLayer geologyLayer = new GeologyLayer(order[i].Type, new Autodesk.Revit.DB.XYZ(order[i].Point.X, order[i].Point.Y, z));
                    newList.Add(geologyLayer);
                }
            }

            this.ValueList = newList;
            EndIndex = ValueList.Count - 1;
        }
    }
}
